#include "ppc.h"

#include "m33.h"

#include "framebuffer.h"

PPC::PPC(float hfov, int _w, int _h): a(1.0f, 0.0f, 0.0f), b(0.0f, -1.0f, 0.0f),
	C(0.0f, 0.0f, 0.0f), w(_w), h(_h) {

	float hfovr = hfov * 3.14159f / 180.0f;
	float f = -(float)w / (2.0f *tanf(hfovr / 2.0f));
	c = V3(-(float)w / 2.0f, (float)h / 2.0f, f);

}


int PPC::Project(V3 p, V3& pp) {

	M33 M; 
	M.SetColumn(0, a);
	M.SetColumn(1, b);
	M.SetColumn(2, c);
	V3 q = M.Inverted()*(p - C);
	if (q[2] < 0.0f)
		return 0;
	pp[2] = 1.0f / q[2];
	pp[0] = q[0] / q[2];
	pp[1] = q[1] / q[2];
	return 1;

}

V3 PPC::GetVD() {

	return (a ^ b).UnitVector();

}

float PPC::GetFocalLength() {

	return GetVD()*c;

}

void PPC::ChangeFocalLength(float scf) {

	c = c + GetVD()*(GetFocalLength()*(scf - 1.0f));

}


void PPC::PositionAndOrient(V3 C1, V3 L1, V3 vpv) {

	V3 vd1 = (L1 - C1).UnitVector();
	V3 a1 = (vd1 ^ vpv).UnitVector() * a.Length();
	V3 b1 = (vd1 ^ a1).UnitVector() * b.Length();
	V3 c1 = vd1 * GetFocalLength() - b1 * ((float)h / 2.0f) - a1 * ((float)w / 2.0f);

	C = C1;
	a = a1;
	b = b1;
	c = c1;

}


void PPC::Pan(float angled) {


	V3 dv = (b * -1.0f).UnitVector();
	a = a.RotateThisVectorAboutDirection(dv, angled);
	c = c.RotateThisVectorAboutDirection(dv, angled);

}



void PPC::Visualize(PPC *ppc3, FrameBuffer *fb3, float vf) {

	V3 colv(0.0f, 0.0f, 0.0f);
	fb3->Draw3DPoint(C, colv, ppc3, 7);

	float scf = vf / GetFocalLength();
	fb3->Draw3DSegment(C, colv, C + c * scf, colv, ppc3);
	fb3->Draw3DSegment(C + c * scf, colv, C + (c + a * (float)w)*scf, colv, ppc3);
	fb3->Draw3DSegment(C + (c + a * (float)w)*scf, colv,
		C + (c + a * (float)w + b * (float)h)*scf, colv, ppc3);
	fb3->Draw3DSegment(
		C + (c + a * (float)w + b * (float)h)*scf, colv,
		C + (c +  b * (float)h)*scf, colv,
		ppc3);
	fb3->Draw3DSegment(
		C + (c + b * (float)h)*scf, colv,
		C+c*scf, colv,
		ppc3);

}
